// Set the dimensions and margins of the graph
const margin = { top: 20, right: 30, bottom: 40, left: 90 },
  width = 460 - margin.left - margin.right,
  height = 400 - margin.top - margin.bottom;

// Append the svg object to the body of the page, setting the width and height
const svg = d3
  .select('#my_dataviz')
  .append('svg')
  .attr('width', width + margin.left + margin.right)
  .attr('height', height + margin.top + margin.bottom)
  .append('g') // Append a 'g' element for the chart area
  .attr(
    'transform',
    `translate(${margin.left},${margin.top})`,
  ); // Translate the chart area according to margins

// Read the data from the CSV file
d3.csv(
  'https://raw.githubusercontent.com/PBhavyaSri/bookings/main/hotel_bookings_updated_less.csv',
).then(function (data) {
  // Process the data: group by year and sum up the bookings
  const bookingsByYear = Array.from(
    d3.group(data, (d) => d.Year),
    ([key, array]) => ({
      year: key,
      bookings: d3.sum(
        array,
        (d) => +d['Number of Bookings'],
      ),
    }),
  );

  // Add X axis - a band scale for years
  const x = d3
    .scaleBand()
    .domain(bookingsByYear.map((d) => d.year)) // Set domain to years extracted from data
    .range([0, width]) // Set range from 0 to the width of the graph
    .padding(0.1); // Add some padding between bands
  svg
    .append('g')
    .attr('transform', `translate(0, ${height})`) // Translate axis to the bottom
    .call(d3.axisBottom(x)); // Call the bottom axis with the x scale

  // Add X axis label
  svg
    .append('text')
    .attr('text-anchor', 'end') // Anchor the text at the end for proper alignment
    .attr('x', width / 2 + margin.left) // Center the text in the middle of the width
    .attr('y', height + margin.top + 20) // Position it below the x-axis
    .text('Year'); // Set the text for the label

  // Add Y axis - a linear scale for number of bookings
  const y = d3
    .scaleLinear()
    .domain([0, d3.max(bookingsByYear, (d) => +d.bookings)]) // Set domain from 0 to max number of bookings
    .range([height, 0]); // Set range from height to 0 (top of the graph)
  svg.append('g').call(d3.axisLeft(y)); // Call the left axis with the y scale

  // Add Y axis label
  svg
    .append('text')
    .attr('text-anchor', 'end') // Anchor the text at the end
    .attr('transform', 'rotate(-90)') // Rotate the label for vertical alignment
    .attr('y', -margin.left + 20) // Position it to the left of the y-axis
    .attr('x', -margin.top - height / 2 + 20) // Center the text in the middle of the height
    .text('Number of Bookings'); // Set the text for the label

  // Add chart title
  svg
    .append('text')
    .attr('x', width / 2) // Center the title in the middle of the width
    .attr('y', 0 - margin.top / 5) // Position it above the chart
    .attr('text-anchor', 'middle') // Center the text
    .style('font-size', '16px') // Set the font size
    .style('text-decoration', 'underline') // Underline the title
    .text('Number of Bookings by Year'); // Set the text for the title

  // Define the line path generator for the bookings
  const lineGenerator = d3
    .line()
    .x((d) => x(d.year)) // Set the x-value as the center of the band for the year
    .y((d) => y(d.bookings)); // Set the y-value as the booking amount

  // Draw the line path for the bookings
  svg
    .append('path')
    .datum(bookingsByYear) // Bind data to the path
    .attr('fill', 'none') // Set fill to none
    .attr('stroke', 'steelblue') // Set the stroke color to steelblue
    .attr('stroke-width', 1.5) // Set the stroke width
    .attr('d', lineGenerator); // Set the 'd' attribute using the line path generator

  // Add the points (circles) for each data point
  svg
    .selectAll('dot')
    .data(bookingsByYear) // Bind data
    .enter()
    .append('circle') // Enter the data and append circles
    .attr('cx', (d) => x(d.year)) // Set the x-position
    .attr('cy', (d) => y(d.bookings)) // Set the y-position
    .attr('r', 5) // Set the radius of the circle
    .attr('fill', 'steelblue'); // Set the fill color
});
